/****************************************************************************
**
** Copyright AbouZakaria.
** All rights reserved.
** Contact: AbouZakaria (yahia67@live.com)
**
** $xbook_BEGIN_LICENSE:GPL$
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** AbouZakaria (yahia67@live.com)
** $xbook_END_LICENSE$
**
****************************************************************************/

#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QFileDialog>
#include <QTextCodec>
#include <QXmlStreamReader>
#include <QMessageBox>
#include <QDomDocument>
#include <QProcess>
#include <QTextStream>
#include <QSettings>
#include <QFontDialog>
MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    Xbook = new xbook();
    setupUi();
    Xbook->view=ui->treeWidget;
    Xbook->textEdit=ui->textEdit;
    loadLayout();
    Xbook->rowsCount=-1;
    Xbook->currentPosition=0;
    QString initialFile = ":/example.html";
    const QStringList args = QCoreApplication::arguments();
    if (args.count() == 2)
        initialFile = args.at(1);

    if (Xbook->load(initialFile)==true){

        this->setWindowTitle(Xbook->infoBookTitle+"/"+Xbook->infoBookAutor);
        ui->actionRecent->setText(Xbook->infoBookTitle);
        emit bookAvailable(true);
    }else{
        ui->textEdit->clear();
        labelStatu->setText("");
        emit bookAvailable(false);
    }
 ui->textEdit->setContextMenuPolicy(Qt::CustomContextMenu);
}

MainWindow::~MainWindow()
{
    Savelayou();
     delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}
void MainWindow::setupUi()
{
    labelStatu=new QLabel;
    ui->frameFind->setVisible(false);
    ui->statusBar->addWidget(labelStatu);
    labelStatu->setVisible(true);
    ui->treeWidget->setColumnHidden(1,true);
    ui->menuView->addAction(ui->dockWidget->toggleViewAction());
    ui->dockWidget->toggleViewAction()->setShortcut(QKeySequence(tr("Ctrl+l")));
    ui->menuView->addAction(ui->mainToolBar->toggleViewAction());
    ui->mainToolBar->toggleViewAction()->setShortcut(QKeySequence(tr("Ctrl+i")));

    actionCopy = new QAction(trUtf8("نسخ"), this);
    actionCopy->setShortcuts(QKeySequence::Copy);
    actionCopy->setStatusTip(trUtf8("نسخ النص المحدد الى الحافظة"));
    connect(actionCopy, SIGNAL(triggered()), ui->textEdit, SLOT(copy()));

    actionSelectAll = new QAction(trUtf8("تحديد الكل"), this);
    actionSelectAll->setShortcuts(QKeySequence::SelectAll);

    actionCopy->setEnabled(false);
    //icons
    ui->actionClose->setIcon(style()->standardIcon(QStyle::SP_DialogCloseButton));
    ui->actionFirst->setIcon(style()->standardIcon(QStyle::SP_ArrowUp));
    ui->actionPrev->setIcon(style()->standardIcon(QStyle::SP_ArrowForward));
    ui->actionNext->setIcon(style()->standardIcon(QStyle::SP_ArrowBack));
    ui->actionLast->setIcon(style()->standardIcon(QStyle::SP_ArrowDown));
    ui->actionOpen->setIcon(style()->standardIcon(QStyle::SP_DirOpenIcon));
    ui->actionRecent->setIcon(style()->standardIcon(QStyle::SP_BrowserReload));
    ui->toolCloseFind->setIcon(style()->standardIcon(QStyle::SP_DialogCloseButton));
    ui->pushButtonFindNext->setIcon(style()->standardIcon(QStyle::SP_ArrowBack));
    ui->actionBookInfo->setIcon(style()->standardIcon(QStyle::SP_MessageBoxInformation));

    connect(this, SIGNAL(bookAvailable(bool)),ui->actionNext, SLOT(setEnabled(bool)));
    connect(this, SIGNAL(bookAvailable(bool)),ui->actionPrev, SLOT(setEnabled(bool)));
    connect(this, SIGNAL(bookAvailable(bool)),ui->actionFirst, SLOT(setEnabled(bool)));
    connect(this, SIGNAL(bookAvailable(bool)),ui->actionLast, SLOT(setEnabled(bool)));
    connect(this, SIGNAL(bookAvailable(bool)),ui->actionBookInfo, SLOT(setEnabled(bool)));
    connect(this, SIGNAL(bookAvailable(bool)),ui->actionArchive, SLOT(setEnabled(bool)));
    connect(this, SIGNAL(bookAvailable(bool)),actionSelectAll, SLOT(setEnabled(bool)));
    connect(ui->textEdit, SIGNAL(copyAvailable(bool)),actionCopy, SLOT(setEnabled(bool)));
    connect(actionSelectAll, SIGNAL(triggered()), ui->textEdit, SLOT(selectAll()));
}

void MainWindow::on_actionOpen_triggered()
{
    QString fn = QFileDialog::getOpenFileName(this, tr("Open File..."),
                                              Xbook->fileName, trUtf8("الملفات المدعومة (*.info *.tar.gz *.bok *.krts);;bok (*.bok)"
                                                                      ";;info (*.info);;krts (*.krts);;tar.gz (*.tar.gz);;All Files (*)" ));
    if (!fn.isEmpty())

    Xbook->currentPosition=0;
    if (Xbook->load(fn)==true){

        this->setWindowTitle(Xbook->infoBookTitle+"/"+Xbook->infoBookAutor);
        ui->actionRecent->setText(Xbook->infoBookTitle);
        emit bookAvailable(true);
    }
}

void MainWindow::on_actionNext_triggered()
{
   if (Xbook->rowsCount==-1)
       return;
    if (Xbook->currentPosition<Xbook->rowsCount)
    Xbook->moveToPosition(Xbook->currentPosition+1);

}

void MainWindow::on_actionPrev_triggered()
{
    if (Xbook->rowsCount==-1)
        return;
    if (Xbook->currentPosition>0)
    Xbook->moveToPosition(Xbook->currentPosition-1);

}

void MainWindow::on_actionFirst_triggered()
{
    if (Xbook->rowsCount==-1)
        return;
     Xbook->moveToPosition(0);

}

void MainWindow::on_actionLast_triggered()
{
    if (Xbook->rowsCount==-1)
        return;
     Xbook->moveToPosition(Xbook->rowsCount);

}

void MainWindow::on_actionRecent_triggered()
{
    Xbook->currentPosition=Xbook->firstPosition;

    if (Xbook->load(Xbook->fileName+"bookinfo.info")==true){

      this->setWindowTitle(Xbook->infoBookTitle+"/"+Xbook->infoBookAutor);
      ui->actionRecent->setText(Xbook->infoBookTitle);
  Xbook->firstPosition=0;
    emit bookAvailable(true);
  }
}

void MainWindow::on_actionFont_triggered()
{
    QFont font;
    font.fromString(Xbook->fontName);
    bool ok = false;
    QFont police =QFontDialog::getFont(&ok,font, this,trUtf8("اختيار خط الصفحة"));
    if (ok)
    {
        Xbook->fontName=(police.toString());
        ui->textEdit->setFont(police);
        Xbook->moveToPosition(Xbook->currentPosition);

    }
}

void MainWindow::on_treeWidget_itemActivated(QTreeWidgetItem* item, int )
{
    Xbook->moveToPosition(item->data(1,1).toInt()-1);

   int index=ui->textEdit->toPlainText().indexOf(item->text(0));
   if (index>0){
       QTextCursor cursor(ui->textEdit->document());
       cursor.setPosition(index,QTextCursor::MoveAnchor);
       cursor.movePosition(QTextCursor::Down, QTextCursor::MoveAnchor);
       ui->textEdit->setTextCursor(cursor);
   }
   ui->treeWidget->setCurrentItem(item);
}

void MainWindow::on_treeWidget_itemClicked(QTreeWidgetItem* item, int )
{
    on_treeWidget_itemActivated(item,0);

}

void MainWindow::on_actionTechkile_toggled(bool r )
{
    Xbook->removeTechkile=r;
    Xbook-> moveToPosition(Xbook->currentPosition);
}

void MainWindow::on_actionFind_triggered()
{
    ui->frameFind->setVisible(true);
    ui->lineEditFind->setFocus();
}

void MainWindow::on_toolCloseFind_clicked()
{
     ui->frameFind->setVisible(false);
}

void MainWindow::on_pushButtonFind_clicked()
{
    Xbook->searchInDoc(ui->textEdit->document(),ui->lineEditFind->text());
}

void MainWindow::on_pushButtonFindNext_clicked()
{
    if (Xbook->rowsCount==-1)
        return;
    if (ui->lineEditFind->text().isEmpty()){
        return;
    }
      Xbook->findNext(ui->textEdit->document(),ui->lineEditFind->text());
}

void MainWindow::on_actionFullScreen_toggled(bool full)
{
    if (full==true){
        this->setWindowState(Qt::WindowFullScreen);
    }else{
        this->setWindowState(Qt::WindowNoState);
    }
}

void MainWindow::on_actionAbout_triggered()
{
    QMessageBox msgBox;
    QPixmap   pixmapG(":/img/xbook48.png") ;
    msgBox.setLayoutDirection(Qt::RightToLeft);
    msgBox.setWindowTitle(trUtf8("حول برنامج كتب اكس"));
    msgBox.setText(trUtf8("برنامج كتب اكس"));
    msgBox.setIconPixmap(pixmapG);
    msgBox.setInformativeText(trUtf8("المؤلف : أبو زكريا")+"\n"+trUtf8("الاصدار : 0.1 تجريبي")
                              +"\n"+trUtf8("الرخصة :  gpl3")
                              +"\n"+trUtf8("يتيح لك هذا البرنامج مطالعة كتب الشاملة والقرطاس بمافيها المضغوطة بصيغة ta.gz على نوافذ لينكس") );

    msgBox.exec();
}

void MainWindow::on_actionClose_triggered()
{
    qApp->quit();
}

void MainWindow::on_actionBookInfo_triggered()
{
    QPixmap   pixmapG(":/img/xbook48.png") ;
    QMessageBox msgBox;
    msgBox.setLayoutDirection(Qt::RightToLeft);
    msgBox.setIconPixmap(pixmapG);
    msgBox.setWindowTitle(trUtf8("معلومات عن الكتاب"));
    msgBox.setText(trUtf8("الكتاب : ")+Xbook->infoBookTitle);
    msgBox.setInformativeText(trUtf8("المؤلف : ")+Xbook->infoBookAutor+"\n"+Xbook->infoBookBetaka);
    msgBox.exec();
}
void MainWindow::on_actionArchive_triggered()
{
    ui->statusBar->showMessage(trUtf8("الرجاء الانتظار جاري انشاء الارشيف"),300);

    QString title=Xbook->infoBookTitle;
    title.replace(" ","_");
    QFile file(QDir::homePath()+"/"+title+".krts");
    if (file.exists()){
        QMessageBox::information(this,trUtf8("معلومات"),trUtf8("الكتاب موجود مسبقا"));
        return;
    }
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    if(Xbook->creatArchive()==true){

        QMessageBox::information(this,trUtf8("معلومات"),trUtf8("لقد تم انشاء الارشيف بنجاح")+"\n"+QDir::homePath()+"/"+title+".krts");
    }else{

        QMessageBox::information(this,trUtf8("معلومات"),trUtf8("لا يوجد كتاب لارشفته"));
    }
    QApplication::restoreOverrideCursor();
}

void MainWindow::loadLayout()//تحميل الاعدادات
{
    QByteArray layo_data=QByteArray::fromHex("");
    QSettings settings(QDir::homePath()+"/.kirtasse/xbook.conf",QSettings::IniFormat);

    settings.beginGroup("MainWindow");

    Xbook->firstPosition=settings.value("currentPosition",0).toInt();
    Xbook->fileName=settings.value("fileName",QDir::homePath()).toString();
    ui->actionRecent->setText(settings.value("infoBookTitle","").toString());
    Xbook->infoBookTitle=settings.value("infoBookTitle","").toString();
    Xbook->fontName=(settings.value("fontName","KacstBook,20,-1,5,50,0,0,0,0,0").toString());
    QFont font;
    font.fromString(Xbook->fontName);
    ui->textEdit->setFont(font);
    this->restoreGeometry(settings.value("geo_data").toByteArray());
    this->restoreState( settings.value("layout_data",layo_data).toByteArray());

    settings.endGroup();
}
void MainWindow::Savelayou()//حفظ الاعدادات الى ملف
{
    QSettings settings(QDir::homePath()+"/.kirtasse/xbook.conf",
                       QSettings::IniFormat);
    settings.beginGroup("MainWindow");
    settings.setValue("currentPosition", Xbook->currentPosition);
    settings.setValue("fileName", Xbook->fileName);
    settings.setValue("fontName", Xbook->fontName);
    settings.setValue("infoBookTitle", Xbook->infoBookTitle);
    settings.setValue("geo_data", saveGeometry());
    settings.setValue("layout_data", saveState());
    settings.endGroup();
}

void MainWindow::on_textEdit_textChanged()
{
    labelStatu->setText(Xbook->infoBookTitle+" / "+Xbook->infoBookAutor+" : "+trUtf8("الصفحة :")+Xbook->bookPage +"  "+trUtf8("الجزء :")+Xbook->bookPart);
}

void MainWindow::on_textEdit_customContextMenuRequested()
{
    QMenu menu(this);
    menu.setLayoutDirection(Qt::RightToLeft);
    menu.addAction(ui->actionNext);
    menu.addAction(ui->actionPrev);
    menu.addAction(ui->actionFirst);
    menu.addAction(ui->actionLast);
    menu.addSeparator();
    menu.addAction(actionCopy);
    menu.addAction(actionSelectAll);
    menu.exec(QCursor::pos());
}
